@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
@@ -50,10 +49,25 @@ class_maps: Dict[str, Set[int]] = {}
 all_symbols: Set[int] = set()
 name_map: Dict[int, str] = {}
 word_search_map: DefaultDict[str, Set[int]] = defaultdict(set)
-zwj = 0x200d
+soft_hyphen = 0xad
 flag_codepoints = frozenset(range(0x1F1E6, 0x1F1E6 + 26))
-marks = set(emoji_skin_tone_modifiers) | {zwj} | flag_codepoints
+# See https://github.com/harfbuzz/harfbuzz/issues/169
+marks = set(emoji_skin_tone_modifiers) | flag_codepoints
 not_assigned = set(range(0, sys.maxunicode))
+property_maps: Dict[str, Set[int]] = defaultdict(set)
+
+
+def parse_prop_list() -> None:
+    global marks
+    for line in get_data('ucd/PropList.txt'):
+        if line.startswith('#'):
+            continue
+        cp_or_range, rest = line.split(';', 1)
+        chars = parse_range_spec(cp_or_range.strip())
+        name = rest.strip().split()[0]
+        property_maps[name] |= chars
+    # see https://www.unicode.org/faq/unsup_char.html#3
+    marks |= property_maps['Other_Default_Ignorable_Code_Point']
 
 
 def parse_ucd() -> None:
@@ -97,6 +111,12 @@ def parse_ucd() -> None:
                 marks.add(codepoint)
             elif category.startswith('S'):
                 all_symbols.add(codepoint)
+            elif category == 'Cf':
+                # we add Cf to marks as it contains things like tags and zero
+                # width chars. Not sure if *all* of Cf should be treated as
+                # combining chars, might need to add individual exceptions in
+                # the future.
+                marks.add(codepoint)
 
     with open('nerd-fonts-glyphs.txt') as f:
         for line in f:
@@ -213,7 +233,7 @@ def parse_eaw() -> None:
         if eaw == 'A':
             ambiguous |= chars
             seen |= chars
-        elif eaw == 'W' or eaw == 'F':
+        elif eaw in ('W', 'F'):
             doublewidth |= chars
             seen |= chars
     doublewidth |= set(range(0x3400, 0x4DBF + 1)) - seen
@@ -234,15 +254,15 @@ def get_ranges(items: List[int]) -> Generator[Union[int, Tuple[int, int]], None,
             yield a, b
 
 
-def write_case(spec: Union[Tuple, int], p: Callable) -> None:
+def write_case(spec: Union[Tuple[int, ...], int], p: Callable[..., None]) -> None:
     if isinstance(spec, tuple):
         p('\t\tcase 0x{:x} ... 0x{:x}:'.format(*spec))
     else:
-        p('\t\tcase 0x{:x}:'.format(spec))
+        p(f'\t\tcase 0x{spec:x}:')
 
 
 @contextmanager
-def create_header(path: str, include_data_types: bool = True) -> Generator[Callable, None, None]:
+def create_header(path: str, include_data_types: bool = True) -> Generator[Callable[..., None], None, None]:
     with open(path, 'w') as f:
         p = partial(print, file=f)
         p('// unicode data, built from the unicode standard on:', date.today())
@@ -282,7 +302,7 @@ def gen_emoji() -> None:
 
 def category_test(
     name: str,
-    p: Callable,
+    p: Callable[..., None],
     classes: Iterable[str],
     comment: str,
     use_static: bool = False,
@@ -312,7 +332,7 @@ def category_test(
     p('\treturn false;\n}\n')
 
 
-def codepoint_to_mark_map(p: Callable, mark_map: List[int]) -> Dict[int, int]:
+def codepoint_to_mark_map(p: Callable[..., None], mark_map: List[int]) -> Dict[int, int]:
     p('\tswitch(c) { // {{{')
     rmap = {c: m for m, c in enumerate(mark_map)}
     for spec in get_ranges(mark_map):
@@ -336,10 +356,10 @@ def classes_to_regex(classes: Iterable[str], exclude: str = '') -> Iterable[str]
 
     def as_string(codepoint: int) -> str:
         if codepoint < 256:
-            return r'\x{:02x}'.format(codepoint)
+            return fr'\x{codepoint:02x}'
         if codepoint <= 0xffff:
-            return r'\u{:04x}'.format(codepoint)
-        return r'\U{:08x}'.format(codepoint)
+            return fr'\u{codepoint:04x}'
+        return fr'\U{codepoint:08x}'
 
     for spec in get_ranges(list(chars)):
         if isinstance(spec, tuple):
@@ -354,16 +374,21 @@ def gen_ucd() -> None:
         p('#include "unicode-data.h"')
         category_test(
                 'is_combining_char', p,
-                {c for c in class_maps if c.startswith('M')},
-                'M category (marks)',
-                # See https://github.com/harfbuzz/harfbuzz/issues/169
-                extra_chars=emoji_skin_tone_modifiers | {zwj},
+                (),
+                'Combining and default ignored characters',
+                extra_chars=marks,
                 least_check_return='false'
         )
         category_test(
-            'is_ignored_char', p, 'Cc Cf Cs'.split(),
+            'is_ignored_char', p, 'Cc Cs'.split(),
             'Control characters and non-characters',
-            extra_chars=non_characters, exclude={zwj},
+            extra_chars=non_characters,
+            ascii_range='false'
+        )
+        category_test(
+            'is_non_rendered_char', p, 'Cc Cs Cf'.split(),
+            'Other_Default_Ignorable_Code_Point and soft hyphen',
+            extra_chars=property_maps['Other_Default_Ignorable_Code_Point'] | set(range(0xfe00, 0xfe0f + 1)),
             ascii_range='false'
         )
         category_test('is_word_char', p, {c for c in class_maps if c[0] in 'LN'}, 'L and N categories')
@@ -378,15 +403,19 @@ def gen_ucd() -> None:
         p('combining_type mark_for_codepoint(char_type c) {')
         rmap = codepoint_to_mark_map(p, mark_map)
         p('}\n')
-        with open('kitty/unicode-data.h') as f:
-            unicode_data = f.read()
-        m = re.search(r'^#define VS15 (\d+)', unicode_data, re.M)
-        if m is not None:
-            expected = int(m.group(1))
-        if rmap[0xfe0e] != expected:
-            raise ValueError('The mark for 0xfe0e has changed, you have to update VS15 to {} and VS16 to {} in unicode-data.h'.format(
-                rmap[0xfe0e], rmap[0xfe0f]
-            ))
+        with open('kitty/unicode-data.h', 'r+') as f:
+            raw = f.read()
+            f.seek(0)
+            raw, num = re.subn(
+                r'^// START_KNOWN_MARKS.+?^// END_KNOWN_MARKS',
+                '// START_KNOWN_MARKS\nstatic const combining_type '
+                f'VS15 = {rmap[0xfe0e]}, VS16 = {rmap[0xfe0f]};'
+                '\n// END_KNOWN_MARKS', raw, flags=re.MULTILINE | re.DOTALL)
+            if not num:
+                raise SystemExit('Faile dto patch mark definitions in unicode-data.h')
+            f.truncate()
+            f.write(raw)
+
     with open('kittens/hints/url_regex.py', 'w') as f:
         f.write('# generated by gen-wcwidth.py, do not edit\n\n')
         f.write("url_delimiters = '{}'  # noqa".format(''.join(classes_to_regex(cz, exclude='\n\r'))))
@@ -496,7 +525,7 @@ def gen_names() -> None:
 def gen_wcwidth() -> None:
     seen: Set[int] = set()
 
-    def add(p: Callable, comment: str, chars_: Union[Set[int], FrozenSet[int]], ret: int) -> None:
+    def add(p: Callable[..., None], comment: str, chars_: Union[Set[int], FrozenSet[int]], ret: int) -> None:
         chars = chars_ - seen
         seen.update(chars)
         p(f'\t\t// {comment} ({len(chars)} codepoints)' + ' {{' '{')
@@ -537,6 +566,7 @@ def gen_wcwidth() -> None:
 
 
 parse_ucd()
+parse_prop_list()
 parse_emoji()
 parse_eaw()
 gen_ucd()
