@@ -5,13 +5,13 @@ import sys
 from contextlib import contextmanager
 from enum import Enum, auto
 from functools import wraps
-from typing import (
-    IO, Any, Callable, Dict, Generator, Optional, TypeVar, Union
-)
+from typing import IO, Any, Callable, Dict, Generator, Optional, TypeVar, Union
 
 from kitty.fast_data_types import Color
 from kitty.rgb import color_as_sharp, to_color
-from kitty.typing import GraphicsCommandType, HandlerType, ScreenSize
+from kitty.typing import (
+    GraphicsCommandType, HandlerType, ScreenSize, UnderlineLiteral
+)
 
 from .operations_stub import CMD
 
@@ -47,6 +47,7 @@ class Mode(Enum):
     ALTERNATE_SCREEN = 1049, '?'
     BRACKETED_PASTE = 2004, '?'
     PENDING_UPDATE = 2026, '?'
+    HANDLE_TERMIOS_SIGNALS = 19997, '?'
 
 
 def cmd(f: F) -> F:
@@ -163,14 +164,11 @@ def set_scrolling_region(screen_size: Optional['ScreenSize'] = None, top: Option
 
 @cmd
 def scroll_screen(amt: int = 1) -> str:
-    return '\033[' + str(abs(amt)) + ('T' if amt < 0 else 'S')
+    return f'\033[{abs(amt)}{"T" if amt < 0 else "S"}'
 
 
-STANDARD_COLORS = {name: i for i, name in enumerate(
-    'black red green yellow blue magenta cyan gray'.split())}
-STANDARD_COLORS['white'] = STANDARD_COLORS['gray']
-UNDERLINE_STYLES = {name: i + 1 for i, name in enumerate(
-    'straight double curly'.split())}
+STANDARD_COLORS = {'black': 0, 'red': 1, 'green': 2, 'yellow': 3, 'blue': 4, 'magenta': 5, 'cyan': 6, 'gray': 7, 'white': 7}
+UNDERLINE_STYLES = {'straight': 1, 'double': 2, 'curly': 3, 'dotted': 4, 'dashed': 5}
 
 
 ColorSpec = Union[int, str, Color]
@@ -217,7 +215,7 @@ def styled(
     bg_intense: bool = False,
     italic: Optional[bool] = None,
     bold: Optional[bool] = None,
-    underline: Optional[str] = None,
+    underline: Optional[UnderlineLiteral] = None,
     underline_color: Optional[ColorSpec] = None,
     reverse: Optional[bool] = None,
     dim: Optional[bool] = None,
@@ -416,6 +414,11 @@ def restore_colors() -> str:
     return '\x1b[#Q'
 
 
+@cmd
+def overlay_ready() -> str:
+    return '\x1bP@kitty-overlay-ready|\x1b\\'
+
+
 @cmd
 def write_to_clipboard(data: Union[str, bytes], use_primary: bool = False) -> str:
     from base64 import standard_b64encode
@@ -468,7 +471,7 @@ def as_type_stub() -> str:
     for name, func in all_cmds.items():
         args = ', '.join(func_sig(func))
         if args:
-            args = ', ' + args
+            args = f', {args}'
         methods.append(f'    def {name}(self{args}) -> str: pass')
     ans += ['', '', 'class CMD:'] + methods
 
