@@ -5,11 +5,10 @@
  * Distributed under terms of the GPL3 license.
  */
 
-#define EXTRA_INIT if (PyModule_AddFunctions(module, module_methods) != 0) return false;
 #include "state.h"
 #include <structmember.h>
+#include "colors.h"
 
-PyTypeObject ColorProfile_Type;
 
 static uint32_t FG_BG_256[256] = {
     0x000000,  // 0
@@ -61,7 +60,7 @@ PyObject* create_256_color_table() {
 }
 
 static PyObject *
-new(PyTypeObject *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
+new_cp(PyTypeObject *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
     ColorProfile *self;
 
     self = (ColorProfile *)type->tp_alloc(type, 0);
@@ -78,14 +77,14 @@ new(PyTypeObject *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
 }
 
 static void
-dealloc(ColorProfile* self) {
+dealloc_cp(ColorProfile* self) {
     if (self->color_stack) free(self->color_stack);
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 ColorProfile*
 alloc_color_profile() {
-    return (ColorProfile*)new(&ColorProfile_Type, NULL, NULL);
+    return (ColorProfile*)new_cp(&ColorProfile_Type, NULL, NULL);
 }
 
 
@@ -114,7 +113,7 @@ copy_color_profile(ColorProfile *dest, ColorProfile *src) {
 static void
 patch_color_table(const char *key, PyObject *profiles, PyObject *spec, size_t which, int change_configured) {
     PyObject *v = PyDict_GetItemString(spec, key);
-    if (v) {
+    if (v && PyLong_Check(v)) {
         color_type color = PyLong_AsUnsignedLong(v);
         for (Py_ssize_t j = 0; j < PyTuple_GET_SIZE(profiles); j++) {
             ColorProfile *self = (ColorProfile*)PyTuple_GET_ITEM(profiles, j);
@@ -128,7 +127,7 @@ patch_color_table(const char *key, PyObject *profiles, PyObject *spec, size_t wh
 
 #define patch_mark_color(key, profiles, spec, array, i) { \
     PyObject *v = PyDict_GetItemString(spec, key); \
-    if (v) { \
+    if (v && PyLong_Check(v)) { \
         color_type color = PyLong_AsUnsignedLong(v); \
         for (Py_ssize_t j = 0; j < PyTuple_GET_SIZE(profiles); j++) { \
             ColorProfile *self = (ColorProfile*)PyTuple_GET_ITEM(profiles, j); \
@@ -139,8 +138,8 @@ patch_color_table(const char *key, PyObject *profiles, PyObject *spec, size_t wh
 
 static PyObject*
 patch_color_profiles(PyObject *module UNUSED, PyObject *args) {
-    PyObject *spec, *profiles, *v; ColorProfile *self; int change_configured; PyObject *cursor_text_color;
-    if (!PyArg_ParseTuple(args, "O!OO!p", &PyDict_Type, &spec, &cursor_text_color, &PyTuple_Type, &profiles, &change_configured)) return NULL;
+    PyObject *spec, *profiles, *v; ColorProfile *self; int change_configured;
+    if (!PyArg_ParseTuple(args, "O!O!p", &PyDict_Type, &spec, &PyTuple_Type, &profiles, &change_configured)) return NULL;
     char key[32] = {0};
     for (size_t i = 0; i < arraysz(FG_BG_256); i++) {
         snprintf(key, sizeof(key) - 1, "color%zu", i);
@@ -151,67 +150,68 @@ patch_color_profiles(PyObject *module UNUSED, PyObject *args) {
     S(background, i); S(foreground, i);
 #undef S
     }
+#define SI(profile_name) \
+    DynamicColor color; \
+    if (PyLong_Check(v)) { \
+        color.rgb = PyLong_AsUnsignedLong(v);  color.type = COLOR_IS_RGB; \
+    } else { color.rgb = 0; color.type = COLOR_IS_SPECIAL; }\
+    self->overridden.profile_name = color; \
+    if (change_configured) self->configured.profile_name = color; \
+    self->dirty = true;
+
 #define S(config_name, profile_name) { \
     v = PyDict_GetItemString(spec, #config_name); \
     if (v) { \
-        color_type color = PyLong_AsUnsignedLong(v); \
         for (Py_ssize_t i = 0; i < PyTuple_GET_SIZE(profiles); i++) { \
             self = (ColorProfile*)PyTuple_GET_ITEM(profiles, i); \
-            self->overridden.profile_name = (color << 8) | 2; \
-            if (change_configured) self->configured.profile_name = color; \
-            self->dirty = true; \
+            SI(profile_name); \
         } \
     } \
 }
         S(foreground, default_fg); S(background, default_bg); S(cursor, cursor_color);
         S(selection_foreground, highlight_fg); S(selection_background, highlight_bg);
+        S(cursor_text_color, cursor_text_color); S(visual_bell_color, visual_bell_color);
+#undef SI
 #undef S
-    if (cursor_text_color != Py_False) {
-        for (Py_ssize_t i = 0; i < PyTuple_GET_SIZE(profiles); i++) {
-            self = (ColorProfile*)PyTuple_GET_ITEM(profiles, i);
-            self->overridden.cursor_text_color = 0x111111;
-            self->overridden.cursor_text_uses_bg = 3;
-            if (cursor_text_color != Py_None) {
-                self->overridden.cursor_text_color = (PyLong_AsUnsignedLong(cursor_text_color) << 8) | 2;
-                self->overridden.cursor_text_uses_bg = 1;
-            }
-            if (change_configured) {
-                self->configured.cursor_text_color = self->overridden.cursor_text_color;
-                self->configured.cursor_text_uses_bg = self->overridden.cursor_text_uses_bg;
-            }
-            self->dirty = true;
-        }
-    }
-
     Py_RETURN_NONE;
 }
 
-color_type
-colorprofile_to_color(ColorProfile *self, color_type entry, color_type defval) {
-    color_type t = entry & 0xFF, r;
-    switch(t) {
-        case 1:
-            r = (entry >> 8) & 0xff;
-            return self->color_table[r];
-        case 2:
-            return entry >> 8;
-        default:
+DynamicColor
+colorprofile_to_color(ColorProfile *self, DynamicColor entry, DynamicColor defval) {
+    switch(entry.type) {
+        case COLOR_NOT_SET:
             return defval;
+        case COLOR_IS_INDEX: {
+            DynamicColor ans;
+            ans.rgb = self->color_table[entry.rgb & 0xff] & 0xffffff;
+            ans.type = COLOR_IS_RGB;
+            return ans;
+        }
+        case COLOR_IS_RGB:
+        case COLOR_IS_SPECIAL:
+            return entry;
     }
+    return entry;
 }
 
-float
-cursor_text_as_bg(ColorProfile *self) {
-    if (self->overridden.cursor_text_uses_bg & 1) {
-        return self->overridden.cursor_text_uses_bg & 2 ? 1.f : 0.f;
+color_type
+colorprofile_to_color_with_fallback(ColorProfile *self, DynamicColor entry, DynamicColor defval, DynamicColor fallback, DynamicColor fallback_defval) {
+    switch(entry.type) {
+        case COLOR_NOT_SET:
+        case COLOR_IS_SPECIAL:
+            if (defval.type == COLOR_IS_SPECIAL) return colorprofile_to_color(self, fallback, fallback_defval).rgb;
+            return defval.rgb;
+        case COLOR_IS_RGB:
+            return entry.rgb;
+        case COLOR_IS_INDEX:
+            return self->color_table[entry.rgb & 0xff] & 0xffffff;
     }
-    return self->configured.cursor_text_uses_bg & 2 ? 1.f : 0.f;
+    return entry.rgb;
 }
 
-
 static PyObject*
 as_dict(ColorProfile *self, PyObject *args UNUSED) {
-#define as_dict_doc "Return all colors as a dictionary of color_name to integer (names are the same as used in kitty.conf)"
+#define as_dict_doc "Return all colors as a dictionary of color_name to integer or None (names are the same as used in kitty.conf)"
     PyObject *ans = PyDict_New();
     if (ans == NULL) return PyErr_NoMemory();
     for (unsigned i = 0; i < arraysz(self->color_table); i++) {
@@ -224,17 +224,22 @@ as_dict(ColorProfile *self, PyObject *args UNUSED) {
         if (ret != 0) { Py_CLEAR(ans); return NULL; }
     }
 #define D(attr, name) { \
-    color_type c = colorprofile_to_color(self, self->overridden.attr, 0xffffffff); \
-    if (c != 0xffffffff) { \
-        PyObject *val = PyLong_FromUnsignedLong(c); \
-        if (!val) { Py_CLEAR(ans); return PyErr_NoMemory(); } \
-        int ret = PyDict_SetItemString(ans, #name, val); \
+    if (self->overridden.attr.type != COLOR_NOT_SET) { \
+        int ret; PyObject *val; \
+        if (self->overridden.attr.type == COLOR_IS_SPECIAL) { \
+            val = Py_None; Py_INCREF(val); \
+        } else { \
+            color_type c = colorprofile_to_color(self, self->overridden.attr, self->configured.attr).rgb; \
+            val = PyLong_FromUnsignedLong(c); \
+        } \
+        if (!val) { Py_CLEAR(ans); return NULL; } \
+        ret = PyDict_SetItemString(ans, #name, val); \
         Py_CLEAR(val); \
         if (ret != 0) { Py_CLEAR(ans); return NULL; } \
     }}
     D(default_fg, foreground); D(default_bg, background);
     D(cursor_color, cursor); D(cursor_text_color, cursor_text); D(highlight_fg, selection_foreground);
-    D(highlight_bg, selection_background);
+    D(highlight_bg, selection_background); D(visual_bell_color, visual_bell_color);
 
 #undef D
     return ans;
@@ -248,7 +253,6 @@ as_color(ColorProfile *self, PyObject *val) {
     unsigned int t = entry & 0xFF;
     uint8_t r;
     uint32_t col = 0;
-    PyObject *ans = NULL;
     switch(t) {
         case 1:
             r = (entry >> 8) & 0xff;
@@ -258,10 +262,11 @@ as_color(ColorProfile *self, PyObject *val) {
             col = entry >> 8;
             break;
         default:
-            ans = Py_None; Py_INCREF(Py_None);
+            Py_RETURN_NONE;
     }
-    if (ans == NULL) ans = Py_BuildValue("BBB", (unsigned char)(col >> 16), (unsigned char)((col >> 8) & 0xFF), (unsigned char)(col & 0xFF));
-    return ans;
+    Color *ans = PyObject_New(Color, &Color_Type);
+    if (ans) ans->color.rgb = col;
+    return (PyObject*)ans;
 }
 
 static PyObject*
@@ -295,11 +300,14 @@ set_color(ColorProfile *self, PyObject *args) {
 static PyObject*
 set_configured_colors(ColorProfile *self, PyObject *args) {
 #define set_configured_colors_doc "Set the configured colors"
-    if (!PyArg_ParseTuple(
-                args, "II|IIIII",
-                &(self->configured.default_fg), &(self->configured.default_bg),
-                &(self->configured.cursor_color), &(self->configured.cursor_text_color), &(self->configured.cursor_text_uses_bg),
-                &(self->configured.highlight_fg), &(self->configured.highlight_bg))) return NULL;
+    unsigned int default_fg, default_bg, cursor_color, cursor_text_color, highlight_fg, highlight_bg, visual_bell_color;
+    if (!PyArg_ParseTuple(args, "II|IIIII", &default_fg, &default_bg,
+        &cursor_color, &cursor_text_color, &highlight_fg, &highlight_bg, &visual_bell_color)) return NULL;
+#define S(which) \
+    self->configured.which.rgb = which & 0xffffff; \
+    self->configured.which.type = (which & 0xff000000) ? COLOR_IS_RGB : COLOR_IS_SPECIAL;
+    S(default_fg); S(default_bg); S(cursor_color); S(cursor_text_color); S(highlight_fg); S(highlight_bg); S(visual_bell_color);
+#undef S
     self->dirty = true;
     Py_RETURN_NONE;
 }
@@ -394,8 +402,14 @@ default_color_table(PyObject *self UNUSED, PyObject *args UNUSED) {
 // Boilerplate {{{
 
 #define CGETSET(name) \
-    static PyObject* name##_get(ColorProfile *self, void UNUSED *closure) { return PyLong_FromUnsignedLong(colorprofile_to_color(self, self->overridden.name, self->configured.name));  } \
-    static int name##_set(ColorProfile *self, PyObject *val, void UNUSED *closure) { if (val == NULL) { PyErr_SetString(PyExc_TypeError, "Cannot delete attribute"); return -1; } self->overridden.name = (color_type) PyLong_AsUnsignedLong(val); self->dirty = true; return 0; }
+    static PyObject* name##_get(ColorProfile *self, void UNUSED *closure) { return PyLong_FromUnsignedLong(colorprofile_to_color(self, self->overridden.name, self->configured.name).rgb);  } \
+    static int name##_set(ColorProfile *self, PyObject *v, void UNUSED *closure) { \
+        if (v == NULL) { PyErr_SetString(PyExc_TypeError, "Cannot delete attribute: " #name); return -1; } \
+        unsigned long val = PyLong_AsUnsignedLong(v); \
+        self->overridden.name.rgb = val & 0xffffff; \
+        self->overridden.name.type = (val & 0xff000000) ? COLOR_IS_RGB : COLOR_NOT_SET; \
+        self->dirty = true; return 0; \
+    }
 
 CGETSET(default_fg)
 CGETSET(default_bg)
@@ -403,23 +417,26 @@ CGETSET(cursor_color)
 CGETSET(cursor_text_color)
 CGETSET(highlight_fg)
 CGETSET(highlight_bg)
+CGETSET(visual_bell_color)
+#undef CGETSET
 
-static PyGetSetDef getsetters[] = {
+static PyGetSetDef cp_getsetters[] = {
     GETSET(default_fg)
     GETSET(default_bg)
     GETSET(cursor_color)
     GETSET(cursor_text_color)
     GETSET(highlight_fg)
     GETSET(highlight_bg)
+    GETSET(visual_bell_color)
     {NULL}  /* Sentinel */
 };
 
 
-static PyMemberDef members[] = {
+static PyMemberDef cp_members[] = {
     {NULL}
 };
 
-static PyMethodDef methods[] = {
+static PyMethodDef cp_methods[] = {
     METHOD(update_ansi_color_table, METH_O)
     METHOD(reset_color_table, METH_NOARGS)
     METHOD(as_dict, METH_NOARGS)
@@ -436,14 +453,155 @@ PyTypeObject ColorProfile_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
     .tp_name = "fast_data_types.ColorProfile",
     .tp_basicsize = sizeof(ColorProfile),
-    .tp_dealloc = (destructor)dealloc,
+    .tp_dealloc = (destructor)dealloc_cp,
     .tp_flags = Py_TPFLAGS_DEFAULT,
     .tp_doc = "ColorProfile",
-    .tp_members = members,
-    .tp_methods = methods,
-    .tp_getset = getsetters,
-    .tp_new = new,
+    .tp_members = cp_members,
+    .tp_methods = cp_methods,
+    .tp_getset = cp_getsetters,
+    .tp_new = new_cp,
 };
+// }}}
+
+
+static PyObject *
+new_color(PyTypeObject *type, PyObject *args, PyObject *kwds) {
+    static const char* kwlist[] = {"red", "green", "blue", "alpha", NULL};
+    Color *self;
+    unsigned char r = 0, g = 0, b = 0, a = 0;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|BBBB", (char**)kwlist, &r, &g, &b, &a)) return NULL;
+
+    self = (Color *)type->tp_alloc(type, 0);
+    if (self != NULL) {
+        self->color.r = r; self->color.g = g; self->color.b = b; self->color.a = a;
+    }
+    return (PyObject*) self;
+}
+
+static PyObject*
+color_as_int(Color *self) {
+    return PyLong_FromUnsignedLong(self->color.val);
+}
+
+static PyObject*
+color_truediv(Color *self, PyObject *divisor) {
+    DECREF_AFTER_FUNCTION PyObject *o = PyNumber_Float(divisor);
+    if (o == NULL) return NULL;
+    double r = self->color.r, g = self->color.g, b = self->color.b, a = self->color.a;
+    double d = PyFloat_AS_DOUBLE(o) * 255.;
+    return Py_BuildValue("dddd", r/d, g/d, b/d, a/d);
+}
+
+static PyNumberMethods color_number_methods = {
+    .nb_int = (unaryfunc)color_as_int,
+    .nb_true_divide = (binaryfunc)color_truediv,
+};
+
+#define CGETSET(name) \
+    static PyObject* name##_get(Color *self, void UNUSED *closure) { return PyLong_FromUnsignedLong(self->color.name);  }
+CGETSET(red)
+CGETSET(green)
+CGETSET(blue)
+CGETSET(alpha)
+#undef CGETSET
+
+static PyObject*
+rgb_get(Color *self, void *closure UNUSED) {
+    return PyLong_FromUnsignedLong(self->color.rgb);
+}
+
+static PyObject*
+luminance_get(Color *self, void *closure UNUSED) {
+    return PyFloat_FromDouble(rgb_luminance(self->color));
+}
+
+static PyObject*
+sgr_get(Color* self, void *closure UNUSED) {
+    char buf[32];
+    int sz = snprintf(buf, sizeof(buf), ":2:%u:%u:%u", self->color.r, self->color.g, self->color.b);
+    return PyUnicode_FromStringAndSize(buf, sz);
+}
+
+static PyObject*
+sharp_get(Color* self, void *closure UNUSED) {
+    char buf[32];
+    int sz;
+    if (self->color.alpha) sz = snprintf(buf, sizeof(buf), "#%02x%02x%02x%02x", self->color.a, self->color.r, self->color.g, self->color.b);
+    else sz = snprintf(buf, sizeof(buf), "#%02x%02x%02x", self->color.r, self->color.g, self->color.b);
+    return PyUnicode_FromStringAndSize(buf, sz);
+}
+
+static PyObject*
+color_cmp(PyObject *self, PyObject *other, int op) {
+    if (op != Py_EQ && op != Py_NE) return Py_NotImplemented;
+    if (!PyObject_TypeCheck(other, &Color_Type)) {
+        if (op == Py_EQ) Py_RETURN_FALSE;
+        Py_RETURN_TRUE;
+    }
+    Color *a = (Color*)self, *b = (Color*)other;
+    switch (op) {
+        case Py_EQ: { if (a->color.val == b->color.val) { Py_RETURN_TRUE; } Py_RETURN_FALSE; }
+        case Py_NE: { if (a->color.val != b->color.val) { Py_RETURN_TRUE; } Py_RETURN_FALSE; }
+        default:
+            return Py_NotImplemented;
+    }
+}
+
+static PyGetSetDef color_getsetters[] = {
+    {"rgb", (getter) rgb_get, NULL, "rgb", NULL},
+    {"red", (getter) red_get, NULL, "red", NULL},
+    {"green", (getter) green_get, NULL, "green", NULL},
+    {"blue", (getter) blue_get, NULL, "blue", NULL},
+    {"alpha", (getter) alpha_get, NULL, "alpha", NULL},
+    {"r", (getter) red_get, NULL, "red", NULL},
+    {"g", (getter) green_get, NULL, "green", NULL},
+    {"b", (getter) blue_get, NULL, "blue", NULL},
+    {"a", (getter) alpha_get, NULL, "alpha", NULL},
+    {"luminance", (getter) luminance_get, NULL, "luminance", NULL},
+    {"as_sgr", (getter) sgr_get, NULL, "as_sgr", NULL},
+    {"as_sharp", (getter) sharp_get, NULL, "as_sharp", NULL},
+    {NULL}  /* Sentinel */
+};
+
+static PyObject*
+contrast(Color* self, PyObject *o) {
+    if (!PyObject_TypeCheck(o, &Color_Type)) { PyErr_SetString(PyExc_TypeError, "Not a Color"); return NULL; }
+    Color *other = (Color*) o;
+    return PyFloat_FromDouble(rgb_contrast(self->color, other->color));
+}
+
+static PyMethodDef color_methods[] = {
+    METHODB(contrast, METH_O),
+    {NULL}  /* Sentinel */
+};
+
+
+static PyObject *
+repr(Color *self) {
+    if (self->color.alpha) return PyUnicode_FromFormat("Color(red=%u, green=%u, blue=%u, alpha=%u)", self->color.r, self->color.g, self->color.b, self->color.a);
+    return PyUnicode_FromFormat("Color(%u, %u, %u)", self->color.r, self->color.g, self->color.b);
+}
+
+static Py_hash_t
+color_hash(PyObject *x) {
+    return ((Color*)x)->color.val;
+}
+
+PyTypeObject Color_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    .tp_name = "kitty.fast_data_types.Color",
+    .tp_basicsize = sizeof(Color),
+    .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_doc = "Color",
+    .tp_new = new_color,
+    .tp_getset = color_getsetters,
+    .tp_as_number = &color_number_methods,
+    .tp_methods = color_methods,
+    .tp_repr = (reprfunc)repr,
+    .tp_hash = color_hash,
+    .tp_richcompare = color_cmp,
+};
+
 
 static PyMethodDef module_methods[] = {
     METHODB(default_color_table, METH_NOARGS),
@@ -452,5 +610,18 @@ static PyMethodDef module_methods[] = {
 };
 
 
-INIT_TYPE(ColorProfile)
+int init_ColorProfile(PyObject *module) {\
+    if (PyType_Ready(&ColorProfile_Type) < 0) return 0;
+    if (PyModule_AddObject(module, "ColorProfile", (PyObject *)&ColorProfile_Type) != 0) return 0;
+    Py_INCREF(&ColorProfile_Type);
+
+    if (PyType_Ready(&Color_Type) < 0) return 0;
+    if (PyModule_AddObject(module, "Color", (PyObject *)&Color_Type) != 0) return 0;
+    Py_INCREF(&Color_Type);
+
+    if (PyModule_AddFunctions(module, module_methods) != 0) return false;
+    return 1;
+}
+
+
 // }}}
