@@ -38,6 +38,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     choices = {}
     imports: Set[Tuple[str, str]] = set()
     tc_imports: Set[Tuple[str, str]] = set()
+    ki_imports: 're.Pattern[str]' = re.compile(r'\b((?:kittens|kitty).+?)[,\]]')
 
     def option_type_as_str(x: Any) -> str:
         needs_import = False
@@ -61,6 +62,11 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
         if isinstance(option, MultiOption):
             typ = typ[typ.index('[') + 1:-1]
             typ = typ.replace('Tuple', 'Dict', 1)
+            kq = ki_imports.search(typ)
+            if kq is not None:
+                kqi = kq.group(1)
+                kqim, kqii = kqi.rsplit('.', 1)
+                imports.add((kqim, ''))
         return func, typ
 
     is_mutiple_vars = {}
@@ -119,7 +125,13 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
             if func.__module__ != 'builtins':
                 tc_imports.add((func.__module__, func.__name__))
 
-        defval = repr(func(option.defval_as_string))
+        defval_as_obj = func(option.defval_as_string)
+        if isinstance(defval_as_obj, frozenset):
+            defval = 'frozenset({' + ', '.join(repr(x) for x in sorted(defval_as_obj)) + '})'
+        else:
+            defval = repr(defval_as_obj)
+        if len(defval) > 100:
+            defval += ' # noqa'
         if option.macos_defval is not unset:
             md = repr(func(option.macos_defval))
             defval = f'{md} if is_macos else {defval}'
@@ -304,7 +316,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
 
     t('')
     t('')
-    t(f'actions = frozenset({tuple(defn.actions)!r})')
+    t(f'actions: typing.FrozenSet[str] = frozenset({tuple(defn.actions)!r})')
     t('')
     t('')
     t('def merge_result_dicts(defaults: typing.Dict[str, typing.Any], vals: typing.Dict[str, typing.Any]) -> typing.Dict[str, typing.Any]:')
@@ -341,14 +353,17 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
         for mod, name in imports:
             mmap.setdefault(mod, []).append(name)
         for mod in sorted(mmap):
-            names = sorted(mmap[mod])
-            lines = textwrap.wrap(', '.join(names), 100)
-            if len(lines) == 1:
-                s = lines[0]
+            names = list(filter(None, sorted(mmap[mod])))
+            if names:
+                lines = textwrap.wrap(', '.join(names), 100)
+                if len(lines) == 1:
+                    s = lines[0]
+                else:
+                    s = '\n    '.join(lines)
+                    s = f'(\n    {s}\n)'
+                a(f'from {mod} import {s}')
             else:
-                s = '\n    '.join(lines)
-                s = f'(\n    {s}\n)'
-            a(f'from {mod} import {s}')
+                s = ''
             if add_module_imports and mod not in seen_mods and mod != s:
                 a(f'import {mod}')
                 seen_mods.add(mod)
@@ -375,7 +390,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     return class_def, '\n'.join(preamble + ['', ''] + tc_lines)
 
 
-def generate_c_conversion(loc: str, ctypes: List[Option]) -> str:
+def generate_c_conversion(loc: str, ctypes: List[Union[Option, MultiOption]]) -> str:
     lines: List[str] = []
     basic_converters = {
         'int': 'PyLong_AsLong', 'uint': 'PyLong_AsUnsignedLong', 'bool': 'PyObject_IsTrue',
@@ -422,7 +437,7 @@ def write_output(loc: str, defn: Definition) -> None:
         f.write(f'{tc}\n')
     ctypes = []
     for opt in defn.root_group.iter_all_non_groups():
-        if isinstance(opt, Option) and opt.ctype:
+        if isinstance(opt, (Option, MultiOption)) and opt.ctype:
             ctypes.append(opt)
     if ctypes:
         c = generate_c_conversion(loc, ctypes)
