@@ -33,8 +33,9 @@ typedef struct {
     PyObject *family_name, *full_name, *postscript_name, *path;
 } CTFace;
 PyTypeObject CTFace_Type;
+static CTFontRef window_title_font = nil;
 
-static inline char*
+static char*
 convert_cfstring(CFStringRef src, int free_src) {
 #define SZ 4094
     static char buf[SZ+2] = {0};
@@ -46,7 +47,7 @@ convert_cfstring(CFStringRef src, int free_src) {
 #undef SZ
 }
 
-static inline void
+static void
 init_face(CTFace *self, CTFontRef font, FONTS_DATA_HANDLE fg UNUSED) {
     if (self->hb_font) hb_font_destroy(self->hb_font);
     self->hb_font = NULL;
@@ -61,7 +62,7 @@ init_face(CTFace *self, CTFontRef font, FONTS_DATA_HANDLE fg UNUSED) {
     self->scaled_point_sz = CTFontGetSize(self->ct_font);
 }
 
-static inline CTFace*
+static CTFace*
 ct_face(CTFontRef font, FONTS_DATA_HANDLE fg) {
     CTFace *self = (CTFace *)CTFace_Type.tp_alloc(&CTFace_Type, 0);
     if (self) {
@@ -174,7 +175,7 @@ coretext_all_fonts(PyObject UNUSED *_self) {
     return ans;
 }
 
-static inline unsigned int
+static unsigned int
 glyph_id_for_codepoint_ctfont(CTFontRef ct_font, char_type ch) {
     unichar chars[2] = {0};
     CGGlyph glyphs[2] = {0};
@@ -183,7 +184,7 @@ glyph_id_for_codepoint_ctfont(CTFontRef ct_font, char_type ch) {
     return glyphs[0];
 }
 
-static inline bool
+static bool
 is_last_resort_font(CTFontRef new_font) {
     CFStringRef name = CTFontCopyPostScriptName(new_font);
     CFComparisonResult cr = CFStringCompare(name, CFSTR("LastResort"), 0);
@@ -191,7 +192,7 @@ is_last_resort_font(CTFontRef new_font) {
     return cr == kCFCompareEqualTo;
 }
 
-static inline CTFontRef
+static CTFontRef
 manually_search_fallback_fonts(CTFontRef current_font, CPUCell *cell) {
     CFArrayRef fonts = CTFontCollectionCreateMatchingFontDescriptors(all_fonts_collection());
     CTFontRef ans = NULL;
@@ -220,7 +221,7 @@ manually_search_fallback_fonts(CTFontRef current_font, CPUCell *cell) {
     return ans;
 }
 
-static inline CTFontRef
+static CTFontRef
 find_substitute_face(CFStringRef str, CTFontRef old_font, CPUCell *cpu_cell) {
     // CTFontCreateForString returns the original font when there are combining
     // diacritics in the font and the base character is in the original font,
@@ -296,7 +297,7 @@ get_glyph_width(PyObject *s, glyph_index g) {
     return (int)ceil(bounds.size.width);
 }
 
-static inline float
+static float
 scaled_point_sz(FONTS_DATA_HANDLE fg) {
     return ((fg->logical_dpi_x + fg->logical_dpi_y) / 144.0) * fg->font_sz_in_pts;
 }
@@ -323,13 +324,6 @@ harfbuzz_font_for_face(PyObject* s) {
     return self->hb_font;
 }
 
-static unsigned int
-adjust_ypos(unsigned int pos, unsigned int cell_height, int adjustment) {
-    if (adjustment >= 0) adjustment = MIN(adjustment, (int)pos - 1);
-    else adjustment = MAX(adjustment, (int)pos - (int)cell_height + 1);
-    return pos - adjustment;
-}
-
 void
 cell_metrics(PyObject *s, unsigned int* cell_width, unsigned int* cell_height, unsigned int* baseline, unsigned int* underline_position, unsigned int* underline_thickness, unsigned int* strikethrough_position, unsigned int* strikethrough_thickness) {
     // See https://developer.apple.com/library/content/documentation/StringsTextFonts/Conceptual/TextAndWebiPhoneOS/TypoFeatures/TextSystemFeatures.html
@@ -375,9 +369,6 @@ cell_metrics(PyObject *s, unsigned int* cell_width, unsigned int* cell_height, u
     CTLineGetTypographicBounds(line, &typographic_ascent, &typographic_descent, &typographic_leading);
     *cell_height = MAX(4u, (unsigned int)ceilf(line_height));
     CGFloat bounds_ascent = bounds_without_leading.size.height + bounds_without_leading.origin.y;
-    int baseline_offset = 0;
-    if (OPT(adjust_baseline_px) != 0) baseline_offset = OPT(adjust_baseline_px);
-    else if (OPT(adjust_baseline_frac) != 0) baseline_offset = (int)(*cell_height * OPT(adjust_baseline_frac));
     *baseline = (unsigned int)floor(bounds_ascent + 0.5);
     // Not sure if we should add this to bounds ascent and then round it or add
     // it to already rounded baseline and round again.
@@ -392,12 +383,6 @@ cell_metrics(PyObject *s, unsigned int* cell_width, unsigned int* cell_height, u
     debug("\tline metrics: ascent: %f descent: %f leading: %f\n", typographic_ascent, typographic_descent, typographic_leading);
     debug("\tfont metrics: ascent: %f descent: %f leading: %f underline_position: %f\n", self->ascent, self->descent, self->leading, self->underline_position);
     debug("\tcell_height: %u baseline: %u underline_position: %u strikethrough_position: %u\n", *cell_height, *baseline, *underline_position, *strikethrough_position);
-    if (baseline_offset) {
-        *baseline = adjust_ypos(*baseline, *cell_height, baseline_offset);
-        *underline_position = adjust_ypos(*underline_position, *cell_height, baseline_offset);
-        *strikethrough_position = adjust_ypos(*strikethrough_position, *cell_height, baseline_offset);
-    }
-
     CFRelease(test_frame); CFRelease(path); CFRelease(framesetter);
 
 #undef count
@@ -445,10 +430,12 @@ finalize(void) {
     free(buffers.render_buf); free(buffers.glyphs); free(buffers.boxes); free(buffers.positions); free(buffers.advances);
     memset(&buffers, 0, sizeof(struct RenderBuffers));
     if (all_fonts_collection_data) CFRelease(all_fonts_collection_data);
+    if (window_title_font) CFRelease(window_title_font);
+    window_title_font = nil;
 }
 
 
-static inline void
+static void
 render_color_glyph(CTFontRef font, uint8_t *buf, int glyph_id, unsigned int width, unsigned int height, unsigned int baseline) {
     CGColorSpaceRef color_space = CGColorSpaceCreateDeviceRGB();
     if (color_space == NULL) fatal("Out of memory");
@@ -492,13 +479,8 @@ ensure_render_space(size_t width, size_t height, size_t num_glyphs) {
     }
 }
 
-static inline void
-render_glyphs(CTFontRef font, unsigned int width, unsigned int height, unsigned int baseline, unsigned int num_glyphs) {
-    memset(buffers.render_buf, 0, width * height);
-    CGColorSpaceRef gray_color_space = CGColorSpaceCreateDeviceGray();
-    if (gray_color_space == NULL) fatal("Out of memory");
-    CGContextRef render_ctx = CGBitmapContextCreate(buffers.render_buf, width, height, 8, width, gray_color_space, (kCGBitmapAlphaInfoMask & kCGImageAlphaNone));
-    if (render_ctx == NULL) fatal("Out of memory");
+static void
+setup_ctx_for_alpha_mask(CGContextRef render_ctx) {
     CGContextSetShouldAntialias(render_ctx, true);
     CGContextSetShouldSmoothFonts(render_ctx, true);
     CGContextSetGrayFillColor(render_ctx, 1, 1); // white glyphs
@@ -506,10 +488,20 @@ render_glyphs(CTFontRef font, unsigned int width, unsigned int height, unsigned
     CGContextSetLineWidth(render_ctx, OPT(macos_thicken_font));
     CGContextSetTextDrawingMode(render_ctx, kCGTextFillStroke);
     CGContextSetTextMatrix(render_ctx, CGAffineTransformIdentity);
+}
+
+static void
+render_glyphs(CTFontRef font, unsigned int width, unsigned int height, unsigned int baseline, unsigned int num_glyphs) {
+    memset(buffers.render_buf, 0, width * height);
+    CGColorSpaceRef gray_color_space = CGColorSpaceCreateDeviceGray();
+    if (gray_color_space == NULL) fatal("Out of memory");
+    CGContextRef render_ctx = CGBitmapContextCreate(buffers.render_buf, width, height, 8, width, gray_color_space, (kCGBitmapAlphaInfoMask & kCGImageAlphaNone));
+    CGColorSpaceRelease(gray_color_space);
+    if (render_ctx == NULL) fatal("Out of memory");
+    setup_ctx_for_alpha_mask(render_ctx);
     CGContextSetTextPosition(render_ctx, 0, height - baseline);
     CTFontDrawGlyphs(font, buffers.glyphs, buffers.positions, num_glyphs, render_ctx);
     CGContextRelease(render_ctx);
-    CGColorSpaceRelease(gray_color_space);
 }
 
 StringCanvas
@@ -537,8 +529,89 @@ render_simple_text_impl(PyObject *s, const char *text, unsigned int baseline) {
     return ans;
 }
 
+static bool
+ensure_ui_font(size_t in_height) {
+    static size_t for_height = 0;
+    if (window_title_font) {
+        if (for_height == in_height) return true;
+        CFRelease(window_title_font);
+    }
+    window_title_font = CTFontCreateUIFontForLanguage(kCTFontUIFontWindowTitle, 0.f, NULL);
+    if (!window_title_font) return false;
+    CGFloat line_height = MAX(1, floor(CTFontGetAscent(window_title_font) + CTFontGetDescent(window_title_font) + MAX(0, CTFontGetLeading(window_title_font)) + 0.5));
+    CGFloat pts_per_px = CTFontGetSize(window_title_font) / line_height;
+    CGFloat desired_size = in_height * pts_per_px;
+    if (desired_size != CTFontGetSize(window_title_font)) {
+        CTFontRef sized = CTFontCreateCopyWithAttributes(window_title_font, desired_size, NULL, NULL);
+        CFRelease(window_title_font);
+        window_title_font = sized;
+        if (!window_title_font) return false;
+    }
+    for_height = in_height;
+    return true;
+}
 
-static inline bool
+bool
+cocoa_render_line_of_text(const char *text, const color_type fg, const color_type bg, uint8_t *rgba_output, const size_t width, const size_t height) {
+    CGColorSpaceRef color_space = CGColorSpaceCreateDeviceRGB();
+    if (color_space == NULL) return false;
+    CGContextRef ctx = CGBitmapContextCreate(rgba_output, width, height, 8, 4 * width, color_space, kCGImageAlphaPremultipliedLast | kCGBitmapByteOrderDefault);
+    CGColorSpaceRelease(color_space);
+    if (ctx == NULL) return false;
+    if (!ensure_ui_font(height)) return false;
+
+    CGContextSetShouldAntialias(ctx, true);
+    CGContextSetShouldSmoothFonts(ctx, true);  // sub-pixel antialias
+    CGContextSetRGBFillColor(ctx, ((bg >> 16) & 0xff) / 255.f, ((bg >> 8) & 0xff) / 255.f, (bg & 0xff) / 255.f, 1.f);
+    CGContextFillRect(ctx, CGRectMake(0.0, 0.0, width, height));
+    CGContextSetTextDrawingMode(ctx, kCGTextFill);
+    CGContextSetTextMatrix(ctx, CGAffineTransformIdentity);
+    CGContextSetRGBFillColor(ctx, ((fg >> 16) & 0xff) / 255.f, ((fg >> 8) & 0xff) / 255.f, (fg & 0xff) / 255.f, 1.f);
+    CGContextSetRGBStrokeColor(ctx, ((fg >> 16) & 0xff) / 255.f, ((fg >> 8) & 0xff) / 255.f, (fg & 0xff) / 255.f, 1.f);
+
+    NSAttributedString *str = [[NSAttributedString alloc] initWithString:@(text) attributes:@{(NSString *)kCTFontAttributeName: (__bridge id)window_title_font}];
+    if (!str) { CGContextRelease(ctx); return false; }
+    CTLineRef line = CTLineCreateWithAttributedString((CFAttributedStringRef)str);
+    [str release];
+    if (!line) { CGContextRelease(ctx); return false; }
+    CGFloat ascent, descent, leading;
+    CTLineGetTypographicBounds(line, &ascent, &descent, &leading);
+    CGContextSetTextPosition(ctx, 0, descent);
+    CTLineDraw(line, ctx);
+    CFRelease(line);
+    CGContextRelease(ctx);
+    return true;
+}
+
+uint8_t*
+render_single_ascii_char_as_mask(const char ch, size_t *result_width, size_t *result_height) {
+    if (!ensure_ui_font(*result_height)) { PyErr_SetString(PyExc_RuntimeError, "failed to create UI font"); return NULL; }
+    unichar chars = ch;
+    CGSize local_advances[1];
+    CTFontGetGlyphsForCharacters(window_title_font, &chars, buffers.glyphs, 1);
+    CTFontGetAdvancesForGlyphs(window_title_font, kCTFontOrientationDefault, buffers.glyphs, local_advances, 1);
+    CGRect bounding_box = CTFontGetBoundingRectsForGlyphs(window_title_font, kCTFontOrientationDefault, buffers.glyphs, buffers.boxes, 1);
+
+    size_t width = (size_t)ceilf(bounding_box.size.width);
+    size_t height = (size_t)ceilf(bounding_box.size.height);
+    uint8_t *canvas = calloc(width, height);
+    if (!canvas) { PyErr_NoMemory(); return NULL; }
+    CGColorSpaceRef gray_color_space = CGColorSpaceCreateDeviceGray();
+    if (gray_color_space == NULL) { PyErr_NoMemory(); free(canvas); return NULL; }
+    CGContextRef render_ctx = CGBitmapContextCreate(canvas, width, height, 8, width, gray_color_space, (kCGBitmapAlphaInfoMask & kCGImageAlphaNone));
+    CGColorSpaceRelease(gray_color_space);
+    if (render_ctx == NULL) { PyErr_NoMemory(); free(canvas); return NULL; }
+    setup_ctx_for_alpha_mask(render_ctx);
+    /* printf("origin.y: %f descent: %f ascent: %f height: %zu size.height: %f\n", bounding_box.origin.y, CTFontGetDescent(window_title_font), CTFontGetAscent(window_title_font), height, bounding_box.size.height); */
+    CGContextSetTextPosition(render_ctx, -bounding_box.origin.x, -bounding_box.origin.y);
+    CTFontDrawGlyphs(window_title_font, buffers.glyphs, buffers.positions, 1, render_ctx);
+    CGContextRelease(render_ctx);
+    *result_width = width; *result_height = height;
+    return canvas;
+}
+
+
+static bool
 do_render(CTFontRef ct_font, bool bold, bool italic, hb_glyph_info_t *info, hb_glyph_position_t *hb_positions, unsigned int num_glyphs, pixel *canvas, unsigned int cell_width, unsigned int cell_height, unsigned int num_cells, unsigned int baseline, bool *was_colored, bool allow_resize, FONTS_DATA_HANDLE fg, bool center_glyph) {
     unsigned int canvas_width = cell_width * num_cells;
     ensure_render_space(canvas_width, cell_height, num_glyphs);
