@@ -13,6 +13,9 @@
 #endif
 
 #include "data-types.h"
+#include <sys/socket.h>
+#include <sys/types.h>
+#include <unistd.h>
 #include "cleanup.h"
 #include "safe-wrappers.h"
 #include "control-codes.h"
@@ -189,8 +192,34 @@ locale_is_valid(PyObject *self UNUSED, PyObject *args) {
     Py_RETURN_TRUE;
 }
 
+static PyObject*
+py_getpeereid(PyObject *self UNUSED, PyObject *args) {
+    int fd;
+    if (!PyArg_ParseTuple(args, "i", &fd)) return NULL;
+    uid_t euid = 0; gid_t egid = 0;
+#ifdef __linux__
+    struct ucred cr;
+    socklen_t sz = sizeof(cr);
+    if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, &cr, &sz) != 0) { PyErr_SetFromErrno(PyExc_OSError); return NULL; }
+    euid = cr.uid; egid = cr.gid;
+#else
+    if (getpeereid(fd, &euid, &egid) != 0) { PyErr_SetFromErrno(PyExc_OSError); return NULL; }
+#endif
+    int u = euid, g = egid;
+    return Py_BuildValue("ii", u, g);
+}
+
+#include "docs_ref_map_generated.h"
+
+static PyObject*
+get_docs_ref_map(PyObject *self UNUSED, PyObject *args UNUSED) {
+    return PyBytes_FromStringAndSize(docs_ref_map, sizeof(docs_ref_map));
+}
+
 static PyMethodDef module_methods[] = {
     {"wcwidth", (PyCFunction)wcwidth_wrap, METH_O, ""},
+    {"get_docs_ref_map", (PyCFunction)get_docs_ref_map, METH_NOARGS, ""},
+    {"getpeereid", (PyCFunction)py_getpeereid, METH_VARARGS, ""},
     {"wcswidth", (PyCFunction)wcswidth_std, METH_O, ""},
     {"open_tty", open_tty, METH_VARARGS, ""},
     {"normal_tty", normal_tty, METH_VARARGS, ""},
@@ -217,11 +246,11 @@ static PyMethodDef module_methods[] = {
 
 
 static struct PyModuleDef module = {
-   .m_base = PyModuleDef_HEAD_INIT,
-   .m_name = "fast_data_types",   /* name of module */
-   .m_doc = NULL,
-   .m_size = -1,
-   .m_methods = module_methods
+    .m_base = PyModuleDef_HEAD_INIT,
+    .m_name = "fast_data_types",   /* name of module */
+    .m_doc = NULL,
+    .m_size = -1,
+    .m_methods = module_methods
 };
 
 
@@ -234,6 +263,7 @@ extern int init_Line(PyObject *);
 extern int init_ColorProfile(PyObject *);
 extern int init_Screen(PyObject *);
 extern bool init_fontconfig_library(PyObject*);
+extern bool init_crypto_library(PyObject*);
 extern bool init_desktop(PyObject*);
 extern bool init_fonts(PyObject*);
 extern bool init_glfw(PyObject *m);
@@ -247,6 +277,7 @@ extern bool init_kittens(PyObject *module);
 extern bool init_logging(PyObject *module);
 extern bool init_png_reader(PyObject *module);
 extern bool init_utmp(PyObject *module);
+extern bool init_loop_utils(PyObject *module);
 #ifdef __APPLE__
 extern int init_CoreText(PyObject *);
 extern bool init_cocoa(PyObject *module);
@@ -268,7 +299,6 @@ shift_to_first_set_bit(CellAttrs x) {
     return ans;
 }
 
-
 EXPORTED PyMODINIT_FUNC
 PyInit_fast_data_types(void) {
     PyObject *m;
@@ -315,6 +345,8 @@ PyInit_fast_data_types(void) {
 #endif
     if (!init_fonts(m)) return NULL;
     if (!init_utmp(m)) return NULL;
+    if (!init_loop_utils(m)) return NULL;
+    if (!init_crypto_library(m)) return NULL;
 
     CellAttrs a;
 #define s(name, attr) { a.val = 0; a.attr = 1; PyModule_AddIntConstant(m, #name, shift_to_first_set_bit(a)); }
