@@ -5,7 +5,7 @@ from functools import partial
 from itertools import repeat
 from typing import (
     Any, Dict, Generator, Iterable, Iterator, List, NamedTuple, Optional,
-    Sequence, Tuple
+    Sequence, Tuple, Union
 )
 
 from kitty.borders import BorderColor
@@ -78,9 +78,22 @@ def set_layout_options(opts: Options) -> None:
     lgd.align_top_left = opts.placement_strategy == 'top-left'
 
 
-def calculate_cells_map(bias: Optional[Sequence[float]], number_of_windows: int, number_of_cells: int) -> List[int]:
+def convert_bias_map(bias: Dict[int, float], number_of_windows: int, number_of_cells: int) -> Sequence[float]:
+    cells_per_window, extra = divmod(number_of_cells, number_of_windows)
+    cell_map = list(repeat(cells_per_window, number_of_windows))
+    cell_map[-1] += extra
+    base_bias = [x / number_of_cells for x in cell_map]
+    return distribute_indexed_bias(base_bias, bias)
+
+
+def calculate_cells_map(
+    bias: Union[None, Sequence[float], Dict[int, float]],
+    number_of_windows: int, number_of_cells: int
+) -> List[int]:
+    if isinstance(bias, dict):
+        bias = convert_bias_map(bias, number_of_windows, number_of_cells)
     cells_per_window = number_of_cells // number_of_windows
-    if bias is not None and 1 < number_of_windows == len(bias) and cells_per_window > 5:
+    if bias is not None and number_of_windows > 1 and number_of_windows == len(bias) and cells_per_window > 5:
         cells_map = [int(b * number_of_cells) for b in bias]
         while min(cells_map) < 5:
             maxi, mini = map(cells_map.index, (max(cells_map), min(cells_map)))
@@ -100,11 +113,12 @@ def layout_dimension(
     start_at: int, length: int, cell_length: int,
     decoration_pairs: DecorationPairs,
     left_align: bool = False,
-    bias: Optional[Sequence[float]] = None
+    bias: Union[None, Sequence[float], Dict[int, float]] = None
 ) -> LayoutDimension:
     number_of_windows = len(decoration_pairs)
     number_of_cells = length // cell_length
-    space_needed_for_decorations: int = sum(map(sum, decoration_pairs))
+    dec_vals: Iterable[int] = map(sum, decoration_pairs)
+    space_needed_for_decorations = sum(dec_vals)
     extra = length - number_of_cells * cell_length
     while extra < space_needed_for_decorations:
         number_of_cells -= 1
@@ -196,10 +210,6 @@ def distribute_indexed_bias(base_bias: Sequence[float], index_bias_map: Dict[int
     return normalize_biases(ans)
 
 
-def variable_bias(num_windows: int, candidate: Dict[int, float]) -> Sequence[float]:
-    return distribute_indexed_bias(list(repeat(1/(num_windows), num_windows)), candidate)
-
-
 class Layout:
 
     name: Optional[str] = None
@@ -217,11 +227,14 @@ class Layout:
         self.blank_rects: List[Rect] = []
         self.layout_opts = self.parse_layout_opts(layout_opts)
         assert self.name is not None
-        self.full_name = self.name + ((':' + layout_opts) if layout_opts else '')
+        self.full_name = f'{self.name}:{layout_opts}' if layout_opts else self.name
         self.remove_all_biases()
 
-    def bias_increment_for_cell(self, is_horizontal: bool) -> float:
+    def bias_increment_for_cell(self, all_windows: WindowList, window_id: int, is_horizontal: bool) -> float:
         self._set_dimensions()
+        return self.calculate_bias_increment_for_a_single_cell(all_windows, window_id, is_horizontal)
+
+    def calculate_bias_increment_for_a_single_cell(self, all_windows: WindowList, window_id: int, is_horizontal: bool) -> float:
         if is_horizontal:
             return (lgd.cell_width + 1) / lgd.central.width
         return (lgd.cell_height + 1) / lgd.central.height
@@ -270,9 +283,12 @@ class Layout:
     def move_window_to_group(self, all_windows: WindowList, group: int) -> bool:
         return all_windows.move_window_group(to_group=group)
 
-    def add_window(self, all_windows: WindowList, window: WindowType, location: Optional[str] = None, overlay_for: Optional[int] = None) -> None:
+    def add_window(
+        self, all_windows: WindowList, window: WindowType, location: Optional[str] = None,
+        overlay_for: Optional[int] = None, put_overlay_behind: bool = False
+    ) -> None:
         if overlay_for is not None and overlay_for in all_windows:
-            all_windows.add_window(window, group_of=overlay_for)
+            all_windows.add_window(window, group_of=overlay_for, head_of_group=put_overlay_behind)
             return
         if location == 'neighbor':
             location = 'after'
@@ -327,7 +343,7 @@ class Layout:
     def xlayout(
         self,
         groups: Iterator[WindowGroup],
-        bias: Optional[Sequence[float]] = None,
+        bias: Union[None, Sequence[float], Dict[int, float]] = None,
         start: Optional[int] = None,
         size: Optional[int] = None,
         offset: int = 0,
@@ -346,7 +362,7 @@ class Layout:
     def ylayout(
         self,
         groups: Iterator[WindowGroup],
-        bias: Optional[Sequence[float]] = None,
+        bias: Union[None, Sequence[float], Dict[int, float]] = None,
         start: Optional[int] = None,
         size: Optional[int] = None,
         offset: int = 0,
