@@ -9,6 +9,7 @@
 #include "unicode-data.h"
 #include "lineops.h"
 #include "charsets.h"
+#include "wcwidth-std.h"
 
 extern PyTypeObject Cursor_Type;
 
@@ -300,24 +301,50 @@ write_hyperlink(hyperlink_id_type hid, ANSIBuf *output) {
 #undef W
 }
 
+static void
+write_mark(const char *mark, ANSIBuf *output) {
+#define W(c) output->buf[output->len++] = c
+    W(0x1b); W(']'); W('1'); W('3'); W('3'); W(';');
+    for (size_t i = 0; mark[i] != 0 && i < 32; i++) W(mark[i]);
+    W(0x1b); W('\\');
+#undef W
 
-void
-line_as_ansi(Line *self, ANSIBuf *output, const GPUCell** prev_cell) {
+}
+
+bool
+line_as_ansi(Line *self, ANSIBuf *output, const GPUCell** prev_cell, index_type start_at, index_type stop_before, char_type prefix_char) {
 #define ENSURE_SPACE(extra) ensure_space_for(output, buf, Py_UCS4, output->len + extra, capacity, 2048, false);
-#define WRITE_SGR(val) { ENSURE_SPACE(128); write_sgr(val, output); }
+#define WRITE_SGR(val) { ENSURE_SPACE(128); escape_code_written = true; write_sgr(val, output); }
 #define WRITE_CH(val) { ENSURE_SPACE(1); output->buf[output->len++] = val; }
-#define WRITE_HYPERLINK(val) { ENSURE_SPACE(2256); write_hyperlink(val, output); }
+#define WRITE_HYPERLINK(val) { ENSURE_SPACE(2256); escape_code_written = true; write_hyperlink(val, output); }
+#define WRITE_MARK(val) { ENSURE_SPACE(64); escape_code_written = true; write_mark(val, output); }
+    bool escape_code_written = false;
     output->len = 0;
-    index_type limit = xlimit_for_line(self);
-    if (limit == 0) return;
+    index_type limit = MIN(stop_before, xlimit_for_line(self));
     char_type previous_width = 0;
+    if (prefix_char) { WRITE_CH(prefix_char); previous_width = wcwidth_std(prefix_char); }
+
+    switch (self->attrs.prompt_kind) {
+        case UNKNOWN_PROMPT_KIND:
+            break;
+        case PROMPT_START:
+            WRITE_MARK("A");
+            break;
+        case SECONDARY_PROMPT:
+            WRITE_MARK("A;k=s");
+            break;
+        case OUTPUT_START:
+            WRITE_MARK("C");
+            break;
+    }
+    if (limit <= start_at) return escape_code_written;
 
     static const GPUCell blank_cell = { 0 };
     GPUCell *cell;
     if (*prev_cell == NULL) *prev_cell = &blank_cell;
     const CellAttrs mask_for_sgr = {.val=SGR_MASK};
 
-    for (index_type pos=0; pos < limit; pos++) {
+    for (index_type pos=start_at; pos < limit; pos++) {
         char_type ch = self->cpu_cells[pos].ch;
         if (ch == 0) {
             if (previous_width == 2) { previous_width = 0; continue; }
@@ -352,12 +379,14 @@ line_as_ansi(Line *self, ANSIBuf *output, const GPUCell** prev_cell) {
         }
         previous_width = cell->attrs.width;
     }
+    return escape_code_written;
 #undef CMP_ATTRS
 #undef CMP
 #undef WRITE_SGR
 #undef WRITE_CH
 #undef ENSURE_SPACE
 #undef WRITE_HYPERLINK
+#undef WRITE_MARK
 }
 
 static PyObject*
@@ -365,7 +394,7 @@ as_ansi(Line* self, PyObject *a UNUSED) {
 #define as_ansi_doc "Return the line's contents with ANSI (SGR) escape codes for formatting"
     const GPUCell *prev_cell = NULL;
     ANSIBuf output = {0};
-    line_as_ansi(self, &output, &prev_cell);
+    line_as_ansi(self, &output, &prev_cell, 0, self->xnum, 0);
     PyObject *ans = PyUnicode_FromKindAndData(PyUnicode_4BYTE_KIND, output.buf, output.len);
     free(output.buf);
     return ans;
@@ -834,7 +863,7 @@ as_text_generic(PyObject *args, void *container, get_line_func get_line, index_t
             // get around to writing a nice pager kitten.
             // see https://github.com/kovidgoyal/kitty/issues/2381
             prev_cell = NULL;
-            line_as_ansi(line, ansibuf, &prev_cell);
+            line_as_ansi(line, ansibuf, &prev_cell, 0, line->xnum, 0);
             t = PyUnicode_FromKindAndData(PyUnicode_4BYTE_KIND, ansibuf->buf, ansibuf->len);
             if (t && ansibuf->len > 0) APPEND(sgr_reset);
         } else {
