@@ -73,6 +73,13 @@ left_shift_line(Line *line, index_type at, index_type num) {
     }
 }
 
+static inline bool
+line_is_empty(const Line *line) {
+    for (index_type i = 0; i < line->xnum; i++) {
+        if (line->cpu_cells[i].ch != BLANK_CHAR) return false;
+    }
+    return true;
+}
 
 typedef Line*(get_line_func)(void *, int);
 void line_clear_text(Line *self, unsigned int at, unsigned int num, char_type ch);
@@ -84,7 +91,7 @@ void line_add_combining_char(Line *, uint32_t , unsigned int );
 index_type line_url_start_at(Line *self, index_type x);
 index_type line_url_end_at(Line *self, index_type x, bool, char_type, bool);
 bool line_startswith_url_chars(Line*);
-void line_as_ansi(Line *self, ANSIBuf *output, const GPUCell**) __attribute__((nonnull));
+bool line_as_ansi(Line *self, ANSIBuf *output, const GPUCell**, index_type start_at, index_type stop_before, char_type prefix_char) __attribute__((nonnull));
 unsigned int line_length(Line *self);
 size_t cell_as_unicode(CPUCell *cell, bool include_cc, Py_UCS4 *buf, char_type);
 size_t cell_as_unicode_for_fallback(CPUCell *cell, Py_UCS4 *buf);
