@@ -1,13 +1,11 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
-import os
 import re
 from ctypes import POINTER, c_uint, c_void_p, cast
 from typing import Callable, Generator, Iterable, Pattern, Tuple, Union, Sequence
 
-from .constants import config_dir
+from .utils import resolve_custom_file
 
 pointer_to_uint = POINTER(c_uint)
 
@@ -23,7 +21,7 @@ def get_output_variables(left_address: int, right_address: int, color_address: i
     )
 
 
-def marker_from_regex(expression: Union[str, Pattern], color: int, flags: int = re.UNICODE) -> MarkerFunc:
+def marker_from_regex(expression: Union[str, 'Pattern[str]'], color: int, flags: int = re.UNICODE) -> MarkerFunc:
     color = max(1, min(color, 3))
     if isinstance(expression, str):
         pat = re.compile(expression, flags=flags)
@@ -45,8 +43,8 @@ def marker_from_multiple_regex(regexes: Iterable[Tuple[int, str]], flags: int =
     expr = ''
     color_map = {}
     for i, (color, spec) in enumerate(regexes):
-        grp = 'mcg{}'.format(i)
-        expr += '|(?P<{}>{})'.format(grp, spec)
+        grp = f'mcg{i}'
+        expr += f'|(?P<{grp}>{spec})'
         color_map[grp] = color
     expr = expr[1:]
     pat = re.compile(expr, flags=flags)
@@ -88,8 +86,6 @@ def marker_from_spec(ftype: str, spec: Union[str, Sequence[Tuple[int, str]]], fl
     if ftype == 'function':
         import runpy
         assert isinstance(spec, str)
-        path = spec
-        if not os.path.isabs(path):
-            path = os.path.join(config_dir, path)
-        return marker_from_function(runpy.run_path(path, run_name='__marker__')["marker"])  # type: ignore
-    raise ValueError('Unknown marker type: {}'.format(ftype))
+        path = resolve_custom_file(spec)
+        return marker_from_function(runpy.run_path(path, run_name='__marker__')["marker"])
+    raise ValueError(f'Unknown marker type: {ftype}')
