@@ -460,6 +460,9 @@ dispatch_osc(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
             REPORT_COMMAND(screen_pop_dynamic_colors);
             screen_pop_colors(screen, 0);
             break;
+        case 697:
+            REPORT_ERROR("Ignoring OSC 697, typically used by Fig for shell integration");
+            break;
         default:
             REPORT_ERROR("Unknown OSC code: %u", code);
             break;
@@ -1087,6 +1090,7 @@ dispatch_dcs(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
                 } else IF_SIMPLE_PREFIX("ssh|", handle_remote_ssh)
                 } else IF_SIMPLE_PREFIX("ask|", handle_remote_askpass)
                 } else IF_SIMPLE_PREFIX("clone|", handle_remote_clone)
+                } else IF_SIMPLE_PREFIX("edit|", handle_remote_edit)
 #undef IF_SIMPLE_PREFIX
                 } else {
                     REPORT_ERROR("Unrecognized DCS @ code: 0x%x", screen->parser_buf[1]);
