@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -17,13 +16,13 @@ if TYPE_CHECKING:
 class SetTabTitle(RemoteCommand):
 
     '''
-    title+: The new title
-    match: Which tab to change the title of
+    title+/str: The new title
+    match/str: Which tab to change the title of
     '''
 
     short_desc = 'Set the tab title'
     desc = (
-        'Set the title for the specified tab(s). If you use the :option:`kitty @ set-tab-title --match` option'
+        'Set the title for the specified tabs. If you use the :option:`kitty @ set-tab-title --match` option'
         ' the title will be set for all matched tabs. By default, only the tab'
         ' in which the command is run is affected. If you do not specify a title, the'
         ' title of the currently active window in the tab is used.'
@@ -38,6 +37,7 @@ class SetTabTitle(RemoteCommand):
         for tab in self.tabs_for_match_payload(boss, window, payload_get):
             if tab:
                 tab.set_title(payload_get('title'))
+        return None
 
 
 set_tab_title = SetTabTitle()
