@@ -559,3 +559,11 @@ class TestDataTypes(BaseTest):
     def test_notify_identifier_sanitization(self):
         from kitty.notify import sanitize_identifier_pat
         self.ae(sanitize_identifier_pat().sub('', '\x1b\nabc\n[*'), 'abc')
+
+    def test_bracketed_paste_sanitizer(self):
+        from kitty.utils import sanitize_for_bracketed_paste
+        for x in ('\x1b[201~ab\x9b201~cd', '\x1b[201\x1b[201~~ab'):  # ]]]
+            q = sanitize_for_bracketed_paste(x.encode('utf-8'))
+            self.assertNotIn(b'\x1b[201~', q)
+            self.assertNotIn('\x9b201~'.encode('utf-8'), q)
+            self.assertIn(b'ab', q)
