@@ -1,5 +1,4 @@
 #!/usr/bin/env python3
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 from functools import partial
@@ -492,7 +491,7 @@ class TestKeys(BaseTest):
 
         self.ae(enc(), '<0;1;1M')
         self.ae(enc(action=defines.RELEASE), '<0;1;1m')
-        self.ae(enc(action=defines.MOVE), '<35;1;1M')
+        self.ae(enc(action=defines.MOVE, button=0), '<35;1;1M')
         self.ae(enc(action=defines.DRAG), '<32;1;1M')
 
         self.ae(enc(R), '<2;1;1M')
