@@ -4,9 +4,20 @@
 import importlib
 import os
 import shutil
+import sys
 import unittest
-from importlib.resources import contents
-from typing import Callable, Generator, NoReturn, Sequence, Set
+from typing import Callable, Generator, Iterator, NoReturn, Sequence, Set
+
+
+def contents(package: str) -> Iterator[str]:
+    try:
+        if sys.version_info[:2] < (3, 10):
+            raise ImportError('importlib.resources.files() doesnt work with frozen builds on python 3.9')
+        from importlib.resources import files
+    except ImportError:
+        from importlib.resources import contents
+        return iter(contents(package))
+    return (path.name for path in files(package).iterdir())
 
 
 def itertests(suite: unittest.TestSuite) -> Generator[unittest.TestCase, None, None]:
